package zero1hd.rhythmbullet.graphics.meshes;

import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.Mesh;
import com.badlogic.gdx.graphics.OrthographicCamera;
import com.badlogic.gdx.graphics.VertexAttribute;
import com.badlogic.gdx.graphics.VertexAttributes.Usage;
import com.badlogic.gdx.graphics.glutils.ShaderProgram;

public class Parallelogram {
	private Mesh mesh;
	private OrthographicCamera cam;
	private ShaderProgram shader;
	
	public final int POSITION_VARS = 2;
	public final int COLOR_VARS = 1; //Packed color data
	public final int TOTAL_VARS = POSITION_VARS + COLOR_VARS;
	public final int MAX_QUADS = 1;
	public final int MAX_VERTS = 4 * MAX_QUADS;

	private float[] verts = new float[MAX_VERTS * TOTAL_VARS];


	public Parallelogram(OrthographicCamera camera) {
		this.cam = camera;
		
		mesh = new Mesh(true, MAX_VERTS, 0, new VertexAttribute(Usage.Position, POSITION_VARS, "a_position"),
				//still expects 4 color components
				new VertexAttribute(Usage.ColorPacked, 4, "a_color"));
	}
	private int idx = 0;

	public void drawParallelogram(float x, float y, float width, float height, float topOffset, Color color) {
		if (idx >= verts.length) {
			flush();
		}
		
		float c = color.toFloatBits();
		
		verts[idx++] = x;
		verts[idx++] = y;
		verts[idx++] = c;
		
		verts[idx++] = x + width;
		verts[idx++] = y;
		verts[idx++] = c;

		verts[idx++] = x + width + topOffset;
		verts[idx++] = y + height;
		verts[idx++] = c;

		verts[idx++] = x + topOffset;
		verts[idx++] = y + height;
		verts[idx++] = c;
	}

	private void flush() {
		if (idx != 0) {
			mesh.setVertices(verts);
			
			Gdx.gl.glDepthMask(false);
			Gdx.gl.glEnable(GL20.GL_BLEND);
			Gdx.gl.glBlendFunc(GL20.GL_SRC_ALPHA, GL20.GL_ONE_MINUS_SRC_ALPHA);
			
			int vertCount = idx/TOTAL_VARS;
			
			cam.setToOrtho(false, Gdx.graphics.getWidth(), Gdx.graphics.getHeight());
			
			shader.begin();
			
			shader.setUniformMatrix("u_projTrans", cam.combined);
			
			mesh.render(shader, GL20.GL_TRIANGLES, 0, vertCount);
			
			shader.end();
			
			Gdx.gl.glDepthMask(true);
			
			idx = 0;
		}
	}
}
